from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field


class User(BaseModel):
    id: int
    username: str
    date_joined: datetime
    email: str
    is_active: bool
    is_blocked: bool
    is_reseller: bool
    image_url: Optional[str] = Field(default=None)
    currency_id: int
    discount: int = Field(alias="discount_value")
    twitch_id: int
    twitch_login: Optional[str] = Field(default=None)

    @property
    def has_image(self) -> bool:
        return self.image_url is not None and len(self.image_url) > 0


class Balance(BaseModel):
    id: int
    amount: int | float
    currency_id: int
    user_id: int
    formatted_amount: float
    currency: str



from . import Client
from .model.user import User as UserModel, Balance


class User:
    @staticmethod
    def account(client: Client) -> UserModel:
        """
        Returns user account

        See: https://api.reyden-x.com/docs#/User/get_user_v1_user__get

        Parameters:
            client (Client): Instance of Client

        Returns:
            UserModel: UserModel object
        """
        r = client.get("/user/")
        return UserModel(**r)

    @staticmethod
    def balance(client: Client) -> Balance:
        """
        Returns user balance

        See: https://api.reyden-x.com/docs#/User/get_balance_v1_user_balance__get

        Parameters:
            client (Client): Instance of Client

        Returns:
            Balance: Balance object
        """
        r = client.get("/user/balance/")
        return Balance(**r)



import re
from typing import Self
from pydantic import BaseModel, model_validator

from .platform import Platform

_youtube_patterns = (
    re.compile(r"https:\/\/www\.youtube\.com\/@([a-zA-Z\d]+)$"),
    re.compile(r"https:\/\/www\.youtube\.com\/channel\/([a-zA-Z_\d]+)$"),
)


class SmoothGain(BaseModel):
    enabled: bool
    minutes: int


class NewOrderParameters(BaseModel):
    price_id: int
    number_of_views: int
    number_of_viewers: int
    launch_mode: str
    delay_time: int
    smooth_gain: SmoothGain

    @property
    def platform(self) -> str:
        return ""

    @model_validator(mode="after")
    def validate_launch_mode(self) -> Self:
        if self.launch_mode not in ("auto", "manual", "delay"):
            raise ValueError("valid launch mode values are 'auto', 'manual' or 'delay'")
        return self

    @model_validator(mode="after")
    def validate_delay_time(self) -> Self:
        if self.launch_mode == "delay":
            if self.delay_time < 5 or self.delay_time > 240:
                raise ValueError(
                    "the number of minutes for delayed start should be from 5 to 240"
                )
        return self


class TwitchOrder(NewOrderParameters):
    twitch_id: int

    @property
    def platform(self) -> str:
        return Platform.TWITCH.value

    @model_validator(mode="after")
    def validate_twitch_id(self) -> Self:
        if self.twitch_id < 1:
            raise ValueError("twitch id must be greater than zero")
        return self


class YouTubeOrder(NewOrderParameters):
    channel_url: str

    @property
    def platform(self) -> str:
        return Platform.YOUTUBE.value

    @model_validator(mode="after")
    def validate_channel_url(self) -> Self:
        for pattern in _youtube_patterns:
            if re.search(pattern=pattern, string=self.channel_url):
                return self

        raise ValueError("this is not a YouTube link")



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field


class Payment(BaseModel):
    id: int
    created_at: datetime
    updated_at: datetime
    payed_at: Optional[datetime] = Field(default=None)
    order_id: int
    amount: float
    external_id: Optional[str] = Field(default=None)
    receipt: Optional[str] = Field(default=None)



from typing import List

from . import Client
from .model.result import Result
from .model.traffic import Traffic as TrafficModel


class Traffic:
    @staticmethod
    def countries(client: Client):
        """
        Traffic statistics by country

        See: https://api.reyden-x.com/docs#/Traffic/Traffic_statistics_by_country_v1_traffic_countries__get

        Parameters:
            client (Client): Instance of Client

        Returns:
            Result[List[TrafficModel]]: Result object
        """
        r = client.get("/traffic/countries/")
        return Result[List[TrafficModel]](**r)

    @staticmethod
    def languages(client: Client):
        """
        Traffic statistics by language

        See: https://api.reyden-x.com/docs#/Traffic/Traffic_statistics_by_language_v1_traffic_languages__get

        Parameters:
            client (Client): Instance of Client

        Returns:
            Result[List[TrafficModel]]: Result object
        """
        r = client.get("/traffic/languages/")
        return Result[List[TrafficModel]](**r)
    
    @staticmethod
    def devices(client: Client):
        """
        Traffic statistics by language

        See: https://api.reyden-x.com/docs#/Traffic/Traffic_statistics_by_device_type_v1_traffic_devices__get

        Parameters:
            client (Client): Instance of Client

        Returns:
            Result[List[TrafficModel]]: Result object
        """
        r = client.get("/traffic/devices/")
        return Result[List[TrafficModel]](**r)



class InvalidCredentialsError(Exception):
    pass


class UnauthorizedError(Exception):
    pass


class NotFoundError(Exception):
    pass


class MethodNotAllowedError(Exception):
    pass


class TooManyRequestsError(Exception):
    pass


class UnknownError(Exception):
    pass



import os
from typing import Optional, Dict

import httpx

from .exceptions import (
    InvalidCredentialsError,
    UnauthorizedError,
    NotFoundError,
    MethodNotAllowedError,
    TooManyRequestsError,
    UnknownError,
)
from .model.token import Token

BASE_URL = "https://api.reyden-x.com/v1"


class Client:
    __slots__ = (
        "username",
        "password",
        "timeout",
        "token",
    )

    def __init__(
        self,
        email: Optional[str] = None,
        password: Optional[str] = None,
        timeout: Optional[httpx.Timeout] = None,
    ):
        email = email or os.getenv("REYDENX_EMAIL")
        password = password or os.getenv("REYDENX_PASSWORD")

        if not email:
            raise ValueError("Email is required")

        if not password:
            raise ValueError("Password is required")

        self.username = email
        self.password = password
        self.timeout = timeout or httpx.Timeout(10.0)
        self.token = None
        self.auth()

    @property
    def is_authenticated(self) -> bool:
        return self.token is not None and self.token.is_valid

    def auth(self):
        r = httpx.post(
            f"{BASE_URL}/token/",
            data={
                "username": self.username,
                "password": self.password,
            },
            headers={
                "Accept": "application/json",
            },
            timeout=5,
        )
        if r.status_code == httpx.codes.OK:
            self.token = Token(**r.json())
        else:
            self.token = None
            raise InvalidCredentialsError

    def request(
        self, method: str, path: str, payload: Optional[Dict] = None
    ) -> Optional[Dict]:
        if not self.is_authenticated:
            raise UnauthorizedError

        headers = {
            "Authorization": f"Bearer {self.token.access_token}",
            "Accept": "application/json",
        }
        path = f"{BASE_URL}{path}"
        match method:
            case "POST":
                r = httpx.post(
                    path, json=payload, headers=headers, timeout=self.timeout
                )
            case "PATCH":
                r = httpx.patch(
                    path, json=payload, headers=headers, timeout=self.timeout
                )
            case _:
                r = httpx.get(path, headers=headers, timeout=self.timeout)

        match r.status_code:
            case httpx.codes.OK:
                return r.json()
            case httpx.codes.UNAUTHORIZED:
                raise UnauthorizedError
            case httpx.codes.NOT_FOUND:
                raise NotFoundError
            case httpx.codes.METHOD_NOT_ALLOWED:
                raise MethodNotAllowedError
            case httpx.codes.TOO_MANY_REQUESTS:
                raise TooManyRequestsError
            case _:
                raise UnknownError

    def get(self, path: str) -> Optional[Dict]:
        return self.request("GET", path)

    def post(self, path: str, payload: Dict) -> Optional[Dict]:
        return self.request("POST", path, payload)

    def patch(self, path: str, payload: Optional[Dict] = None) -> Optional[Dict]:
        return self.request("PATCH", path, payload)



from datetime import datetime, timezone

from pydantic import BaseModel


class Token(BaseModel):
    access_token: str
    expires_in: datetime

    @property
    def is_valid(self) -> bool:
        return self.expires_in.timestamp() > datetime.now(tz=timezone.utc).timestamp()



from . import Client
from .model.task import ActionResult, TaskStatus
from .model.launch_params import LaunchParams


class Action:
    @staticmethod
    def status(client: Client, order_id: int, task_id: str) -> TaskStatus:
        """
        See: https://api.reyden-x.com/docs#/Orders/order_get_task_status_v1_orders__order_id__task__task_id__status__get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            task_id (str): Task identifier

        Returns:
            TaskStatus: TaskStatus object
        """
        r = client.get(f"/orders/{order_id}/task/{task_id}/status/")
        return TaskStatus(**r)

    @staticmethod
    def run(client: Client, order_id: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/order_run_v1_orders__order_id__action_run__patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/run/")
        return ActionResult(**r)

    @staticmethod
    def stop(client: Client, order_id: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/order_stop_v1_orders__order_id__action_stop__patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/stop/")
        return ActionResult(**r)

    @staticmethod
    def cancel(client: Client, order_id: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/order_cancel_v1_orders__order_id__action_cancel__patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/cancel/")
        return ActionResult(**r)

    @staticmethod
    def change_online_value(client: Client, order_id: int, value: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/order_change_online_v1_orders__order_id__action_change_online__value___patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            value (int): New value

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/change/online/{value}/")
        return ActionResult(**r)

    @staticmethod
    def change_increase_value(
        client: Client, order_id: int, value: int
    ) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/change_increase_value_v1_orders__order_id__action_increase_change__value___patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            value (int): New value

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/increase/change/{value}/")
        return ActionResult(**r)

    @staticmethod
    def increase_on(client: Client, order_id: int, value: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/increase_on_v1_orders__order_id__action_increase_on__value___patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            value (int): New value

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/increase/on/{value}/")
        return ActionResult(**r)

    @staticmethod
    def increase_off(client: Client, order_id: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/increase_off_v1_orders__order_id__action_increase_off__patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/increase/off/")
        return ActionResult(**r)

    @staticmethod
    def add_views(client: Client, order_id: int, value: int) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/add_views_v1_orders__order_id__action_add_views__value___patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            value (int): The number of views to add to the order

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/add/views/{value}/")
        return ActionResult(**r)

    @staticmethod
    def change_launch_mode(
        client: Client, order_id: int, launch_params: LaunchParams
    ) -> ActionResult:
        """
        See: https://api.reyden-x.com/docs#/Orders/change_launch_params_v1_orders__order_id__action_change_launch__patch

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            launch_params (LaunchParams): Launch Parameters

        Returns:
            ActionResult: Result object
        """
        r = client.patch(f"/orders/{order_id}/action/change/launch/", launch_params.model_dump())
        return ActionResult(**r)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field

from .content_type import ContentType
from .online_stat import OnlineStat
from .platform import Platform


class OrderParameters(BaseModel):
    delay: bool
    delay_time: int
    even_distribution: bool
    even_distribution_time: int
    launch_mode: str
    work_mode: str


class Avg(BaseModel):
    online: OnlineStat
    session_in_seconds: int


class OrderStats(BaseModel):
    active_time_in_seconds: int
    clicks: int
    views: int
    ctr: float
    average: Avg


class Order(BaseModel):
    id: int
    created_at: datetime
    updated_at: datetime
    status: str
    ordered_view_qty: int
    price_per_view: float
    is_autostart: bool
    online_users_limit: int
    tariff_id: int
    platform: Platform
    content_type: ContentType
    parameters: OrderParameters
    extras: Optional[dict] = Field(default=None)
    statistics: Optional[OrderStats] = Field(default=None)
    content_classification_labels: list = Field(default=[])



from unittest import TestCase

from pyreydenx import Client
from pyreydenx.exceptions import InvalidCredentialsError


class TestClient(TestCase):
    def test_client_empty_email(self):
        with self.assertRaises(ValueError) as ctx:
            Client("", "")
        self.assertTrue(str(ctx.exception).lower() == "email is required")

    def test_client_empty_password(self):
        with self.assertRaises(ValueError) as ctx:
            Client("email", "")
        self.assertTrue(str(ctx.exception).lower() == "password is required")

    def test_invalid_creds(self):
        with self.assertRaises(InvalidCredentialsError) as ctx:
            Client("email", "password")
        self.assertIsInstance(ctx.exception, InvalidCredentialsError)



from pydantic import BaseModel


class MinMaxStep(BaseModel):
    min: int
    max: int
    step: int


class Price(BaseModel):
    id: int
    name: str
    format: str
    price: float
    description: str
    views: MinMaxStep
    online_viewers: MinMaxStep
    category_id: int



from typing import List, Optional, Union

from . import Client
from .model.date_qty import DateQty
from .model.id_qty import IdQty
from .model.new_order_parameters import TwitchOrder, YouTubeOrder
from .model.online_stat import OnlineStat
from .model.order import Order as OrderModel
from .model.payment import Payment
from .model.result import Result
from .model.site_stat import SiteStat
from .model.task import ActionResult


class Order:
    @staticmethod
    def get_orders(
        client: Client, cursor: Optional[str] = None
    ) -> Result[List[OrderModel]]:
        """
        Returns list of orders

        See: https://api.reyden-x.com/docs#/Orders/orders_v1_orders__get

        Parameters:
            client (Client): Instance of Client
            cursor (str): Optional parameter which allows to get next part of objects if exists.

        Returns:
            Result[List[OrderModel]]: Result object
        """
        if not cursor:
            r = client.get("/orders/")
        else:
            r = client.get(f"/orders/?cursor={cursor}")
        return Result[List[OrderModel]](**r)

    @staticmethod
    def details(client: Client, order_id: int) -> Result[OrderModel]:
        """
        Order details

        See: https://api.reyden-x.com/docs#/Orders/order_details_v1_orders__order_id___get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            Result[OrderModel]: Result object
        """
        r = client.get(f"/orders/{order_id}/")
        return Result[OrderModel](**r)

    @staticmethod
    def payments(
        client: Client, order_id: int, cursor: Optional[str] = None
    ) -> Result[List[Payment]]:
        """
        Returns list of payments for order

        See: https://api.reyden-x.com/docs#/Orders/order_payments_v1_orders__order_id__payments__get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order
            cursor (str): Optional parameter which allows to get next part of objects if exists.

        Returns:
            Result[List[Payment]]: Result object
        """
        if not cursor:
            r = client.get(f"/orders/{order_id}/payments/")
        else:
            r = client.get(f"/orders/{order_id}/payments/?cursor={cursor}")
        return Result[List[Payment]](**r)

    @staticmethod
    def online_stats(client: Client, order_id: int) -> Result[List[OnlineStat]]:
        """
        Detailed information about users online

        See: https://api.reyden-x.com/docs#/Orders/order_stats_online_v1_orders__order_id__statistics_online__get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            Result[List[OnlineStat]]: Result object
        """
        r = client.get(f"/orders/{order_id}/statistics/online/")
        return Result[List[OnlineStat]](**r)

    @staticmethod
    def clicks_stats(client: Client, order_id: int) -> Result[List[DateQty]]:
        """
        Detailed information about clicks

        See: https://api.reyden-x.com/docs#/Orders/order_stats_clicks_v1_orders__order_id__statistics_clicks__get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            Result[List[DateQty]]: Result object
        """
        r = client.get(f"/orders/{order_id}/statistics/clicks/")
        return Result[List[DateQty]](**r)

    @staticmethod
    def views_stats(client: Client, order_id: int) -> Result[List[DateQty]]:
        """
        Detailed information about views

        See: https://api.reyden-x.com/docs#/Orders/order_stats_views_v1_orders__order_id__statistics_views__get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            Result[List[DateQty]]: Result object
        """
        r = client.get(f"/orders/{order_id}/statistics/views/")
        return Result[List[DateQty]](**r)

    @staticmethod
    def sites_stats(client: Client, order_id: int) -> Result[List[SiteStat]]:
        """
        Detailed information about sites

        See: https://api.reyden-x.com/docs#/Orders/order_stats_sites_v1_orders__order_id__statistics_sites__get

        Parameters:
            client (Client): Instance of Client
            order_id (int): Number of order

        Returns:
            Result[List[SiteStat]]: Result object
        """
        r = client.get(f"/orders/{order_id}/statistics/sites/")
        return Result[List[SiteStat]](**r)

    @staticmethod
    def multiple_views_stats(
        client: Client, identifiers: List[int]
    ) -> Result[List[IdQty]]:
        """
        View statistics for multiple orders

        See: https://api.reyden-x.com/docs#/Orders/multiple_views_v1_orders_multiple_views__post

        Parameters:
            client (Client): Instance of Client
            identifiers (List[int]): Identifiers

        Returns:
            Result[List[IdQty]]: Result object
        """
        r = client.post("/orders/multiple/views/", {"identifiers": identifiers[:100]})
        return Result[List[IdQty]](**r)

    @staticmethod
    def multiple_clicks_stats(
        client: Client, identifiers: List[int]
    ) -> Result[List[IdQty]]:
        """
        Click-through statistics for multiple orders

        See: https://api.reyden-x.com/docs#/Orders/multiple_clicks_v1_orders_multiple_clicks__post

        Parameters:
            client (Client): Instance of Client
            identifiers (List[int]): Identifiers

        Returns:
            Result[List[IdQty]]: Result object
        """
        r = client.post("/orders/multiple/clicks/", {"identifiers": identifiers[:100]})
        return Result[List[IdQty]](**r)

    @staticmethod
    def create(
        client: Client, parameters: Union[TwitchOrder, YouTubeOrder]
    ) -> ActionResult:
        """
        Create new order for Twitch or YouTube stream

        See: https://api.reyden-x.com/docs#/Orders/twitch_stream_v1_orders_create_twitch_stream__post
        See: https://api.reyden-x.com/docs#/Orders/youtube_stream_v1_orders_create_youtube_stream__post

        Parameters:
            client (Client): Instance of Client
            parameters (NewOrderParameters): TwitchOrder or YouTubeOrder object

        Returns:
            ActionResult: Result object
        """
        r = client.post(
            f"/orders/create/{parameters.platform}/stream/", parameters.model_dump()
        )
        return ActionResult(**r)



from enum import StrEnum, auto
from typing import Self
from pydantic import BaseModel, Field, model_validator


class LaunchMode(StrEnum):
    AUTO = auto()
    MANUAL = auto()
    DELAY = auto()


class LaunchParams(BaseModel):
    mode: LaunchMode
    delay_time: int = Field(default=0)

    @model_validator(mode="after")
    def delay_time_validator(self) -> Self:
        if self.mode in (LaunchMode.AUTO, LaunchMode.MANUAL):
            self.delay_time = 0

        if self.mode == LaunchMode.DELAY:
            if self.delay_time < 5 or self.delay_time > 240:
                raise ValueError(
                    "the number of minutes for delayed start should be from 5 to 240"
                )

        return self



from datetime import datetime
from enum import StrEnum, auto
from typing import Optional

from pydantic import BaseModel, Field


class Task(BaseModel):
    id: str
    url: str
    expires_at: datetime


class TaskStatusChoices(StrEnum):
    PENDING = auto()
    ERROR = auto()
    IN_PROGRESS = auto()
    COMPLETED = auto()
    ACTION_REQUIRED = auto()


class TaskStatus(BaseModel):
    status: TaskStatusChoices
    details: Optional[dict] = Field(default=None)


class ActionResult(BaseModel):
    request_id: Optional[str]
    order_id: Optional[int]
    action: str
    value: Optional[int]
    task: Task



import os
from distutils.core import setup

from setuptools import find_packages

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

setup(
    name="pyreydenx",
    packages=find_packages(),
    python_requires=">=3.11",
    version="1.0.10",
    license="Apache Software License",
    description="Reyden-X is an automated service for promoting live broadcasts on external "
    "sites with integrated system of viewers and views management.",
    long_description=README,
    author="pixel365",
    author_email="pixel.365.24@gmail.com",
    url="https://github.com/pixel365/pyreydenx",
    download_url="https://github.com/pixel365/pyreydenx/archive/master.zip",
    project_urls={
        "Documentation": "https://api.reyden-x.com/docs",
        "Source": "https://github.com/pixel365/pyreydenx",
    },
    keywords=["reydenx", "twitch", "trovo", "youtube", "vkplay", "goodgame"],
    install_requires=[
        "httpx>=0.20, <0.30",
        "pydantic>=2.0, <3.0",
    ],
    setup_requires=[
        "httpx",
        "pydantic",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3 :: Only",
    ],
)



from datetime import datetime
from typing import Optional, Generic, TypeVar

from pydantic import BaseModel, Field

T = TypeVar("T")


class Result(BaseModel, Generic[T]):
    request_id: str
    cached: bool
    cache_expires_at: Optional[datetime] = Field(default=None)
    cursor: Optional[str] = Field(default=None)
    result: T

    @property
    def has_next(self) -> bool:
        return isinstance(self.cursor, str) and len(self.cursor) > 0



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field


class OnlineStat(BaseModel):
    created_at: Optional[datetime] = Field(default=None)
    in_settings: int
    in_fact: int



from typing import List
from . import Client
from .model.platform import Platform
from .model.price import Price
from .model.price_category import PriceCategory
from .model.result import Result


class Prices:
    @staticmethod
    def get_prices(client: Client, platform: Platform) -> Result[List[Price]]:
        """
        Returns all rates for a specific platform.

        See: https://api.reyden-x.com/docs#/Prices/prices_v1_prices__platform_code___get

        Parameters:
            client (Client): Instance of Client
            platform (Platform): twitch, youtube etc.

        Returns:
            Result[List[Price]]: Result object
        """
        r = client.get(f"/prices/{platform.value}/")
        return Result[List[Price]](**r)

    @staticmethod
    def get_categories(client: Client) -> Result[List[PriceCategory]]:
        """
        Returns all price categories.

        See: https://api.reyden-x.com/docs#/Price%20Categories/categories_v1_price_categories__get

        Parameters:
            client (Client): Instance of Client

        Returns:
            Result[List[PriceCategory]]: Result object
        """
        r = client.get("/price-categories/")
        return Result[List[PriceCategory]](**r)
